﻿/* Downloaded from: http://www.sphinxconnector.net/Tutorial/IntroductionToRealTimeIndexes */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Xml;
using System.Xml.Linq;

using Common.Logging;
using Common.Logging.Simple;

using SphinxConnector.SphinxQL;

namespace RtIndexSample
{
    class Program
    {
        private static Process sphinxProcess;

        static void Main(string[] args)
        {
            //SetupLogging(); //see: http://www.sphinxconnector.net/blog/post/2013/01/18/A-Quick-Way-to-Setup-Logging-during-Development.aspx

            StartSphinx();

            var xmlReader = XmlReader.Create("posts.xml");
            xmlReader.Read();
            xmlReader.MoveToContent();

            InsertPosts(xmlReader);

            DeletePost(4);

            StopSphinx();
        }

        public static void InsertPosts(XmlReader xmlReader)
        {
            var posts = from row in GetChildContentElements(xmlReader)
                        select new
                        {
                            PostId = row.Attribute("Id").Value,
                            PostTypeId = row.Attribute("PostTypeId").Value,
                            Body = row.Attribute("Body").Value,
                            Tags = row.Attribute("Tags") != null ? row.Attribute("Tags").Value : String.Empty,
                            Title = row.Attribute("Title") != null ? row.Attribute("Title").Value : String.Empty,
                        };

            using (var connection = new SphinxQLConnection())
            {
                var command = connection.CreateCommand();

                command.CommandText = "INSERT INTO so_posts (id, title, body, tags, post_type) VALUES (@postId, @title, @body, @tags, @post_type)";
                command.Parameters.Add("postId", 0, SphinxType.UInt);
                command.Parameters.Add("title", String.Empty, SphinxType.String);
                command.Parameters.Add("body", String.Empty, SphinxType.String);
                command.Parameters.Add("tags", String.Empty, SphinxType.String);
                command.Parameters.Add("post_type", String.Empty, SphinxType.UInt);

                connection.Open();

                foreach (var post in posts)
                {
                    command.Parameters["postId"].Value = post.PostId;
                    command.Parameters["title"].Value = post.Title;
                    command.Parameters["body"].Value = post.Body;
                    command.Parameters["tags"].Value = post.Tags;
                    command.Parameters["post_type"].Value = post.PostTypeId;

                    command.ExecuteNonQuery();
                }
            }
        }

        public static void DeletePost(int postId)
        {
            using (var connection = new SphinxQLConnection())
            {
                var command = connection.CreateCommand();

                command.CommandText = "DELETE FROM so_posts WHERE id = @postId";
                command.Parameters.Add("postId", postId);

                connection.Open();

                command.ExecuteNonQuery();
            }
        }

        [Conditional("DEBUG")]
        private static void SetupLogging()
        {
            LogManager.Adapter = new ConsoleOutLoggerFactoryAdapter
            {
                Level = LogLevel.All,
                ShowLevel = true,
                ShowDateTime = true,
            };
        }

        private static void StartSphinx()
        {
            Console.WriteLine("Starting Sphinx...");

            sphinxProcess = Process.Start(new ProcessStartInfo("searchd.exe")
            {
                WorkingDirectory = @"..\..\..\sphinx\"
            });
        }

        private static void StopSphinx()
        {
            sphinxProcess.Kill();
        }

        static IEnumerable<XElement> GetChildContentElements(XmlReader reader)
        {
            while (reader.Read())
            {
                reader.MoveToContent();

                if (reader.NodeType != XmlNodeType.Element)
                {
                    yield break;
                }

                yield return (XElement)XElement.ReadFrom(reader);
            }
        }
    }
}
